package com.jasamedika.medifirst2000.service;

import java.util.List;
import java.util.Map;

import com.jasamedika.medifirst2000.entities.LimbahB3Keluar;
import com.jasamedika.medifirst2000.vo.LimbahB3KeluarVO;

public interface LimbahB3KeluarService <T> extends BaseVoService<LimbahB3Keluar, LimbahB3KeluarVO, String> {

	Map<String, Object> addLimbahB3Keluar(LimbahB3KeluarVO vo);
	
	Map<String, Object> updateLimbahB3Keluar(LimbahB3KeluarVO vo);

	Map<String, Object> findAllLimbahB3Keluar();

	boolean deleteLimbahB3Keluar(String noRec);

	Map<String, Object> findLimbahB3KeluarByPeriode(String periodeAwal, String periodeAkhir);

	Map<String, Object> lisLimbahB3Keluar(Integer page, Integer limit, String sort, String dir, String dateStart,String dateEnd, Integer jenisLimbah);
	
	Map<String, Object> limbahB3KeluarByNoRec(String noRec);
	
	
	Map<String, Object> getLimbahB3Keluar(String periodeAwal, String periodeAkhir);
	
	Map<String, Object> getRekanan();
	
	Map<String, Object> getSisaLimbahdiTpsByJenisLimbah(Integer jenisLimbah);

}
